from djitellopy import Tello
import time
import matplotlib.pyplot as plt
import concurrent.futures
import threading

Ts = 0.008
RC_Z = RC_YAW = 0
seuil = 5
Z = YAW = 0

VRR = 100

Validation = False
Registering = True
Controling = True

running_check = False



#################################
##          Functions          ##
#################################



def check():
    global Validation, running_check
    running_check = True
    while running_check:
        if abs(VRR) < seuil:      
            st = time.time()
            while (time.time() - st) < 2:
                Temporary_validation = False
                if abs(VRR) < seuil:
                    Temporary_validation = True
                else:
                    break
            Validation = Temporary_validation
        time.sleep(Ts)  # run check in every Ts

def go_to_position(waypoints):
    """Go to z coordinates and check that Z is around the wanted value"""

    print("going to position")

    global RC_Z, VRR, Z, Controling, Validation

    Kp_z = 2.04
    Ti_z = 0.4
    Td_z = 0 # 0.174
    Integral_z = 0
    e0_z = e_z = 0
    
    point_index = 0

    thread = None

    if thread is None:  # Only start a new thread if the old one is finished
        thread = threading.Thread(target=check)
        thread.start()

    while point_index < len(waypoints):
            ############ Z ############
        r_z = tello.get_distance_tof()
        Z = waypoints[point_index][2]
        e_z = Z - r_z

        # Update proportional
        Proportional_z = Kp_z * e_z
        # Update integral
        Integral_z += (Kp_z / Ti_z) * Ts * e_z
        # Update derivative
        Derivative_z = Kp_z * Td_z * ((e_z - e0_z)/Ts)
        # Compute command
        rc_z = Proportional_z + Derivative_z
        # Save current error
        e0_z = e_z
            ###########################
        
        print("e_z: " + str(e_z) + "\n")

        rc_z = int(saturation(rc_z, -100, 100))

        RC_Z = rc_z
        
        VRR = e_z

        if Validation == True:
            point_index += 1
            Validation = False

        time.sleep(Ts)
    
def send_rc():
    while Controling == True:
        tello.send_rc_control(0, 0, RC_Z, RC_YAW)
        time.sleep(Ts)

def saturation(val, min, max):
    if val > max:
        val = max
    elif val < min:
        val = min
    return val

def register(start_time):
    print("Registering")
    result = open("result.txt","w")

    distance_z = [0, 0]
    Time = [0, 0]
    speed_z = 0

    while Registering == True:
        instant_time = time.time() - start_time
        Time[0] = Time[1]
        distance_z[0] = distance_z[1]

        Time[1] = instant_time
        distance_z[1] = tello.get_distance_tof()
        
        speed_z = (distance_z[1] - distance_z[0])/(Time[1] - Time[0])
 
        result.write(str(instant_time) + " " + str(tello.get_distance_tof()) + " " + str(RC_Z) + " " + str(Z) + " " + str(-tello.get_speed_z()*10) +"\n")        
        time.sleep(Ts)
    
    result.close()
    
    print("Done registering")



##############################
##          Control         ##
##############################



tello = Tello()
tello.connect()

print("battery: " + str(tello.get_battery()) + "%")

time.sleep(3)

######################################
##      Open Loop - Beginning       ##
with concurrent.futures.ThreadPoolExecutor() as executor:
    start_time = time.time()
    
    executor.map(register, [start_time])  

    tello.takeoff()

    time.sleep(3)

    Controling = False

    running_check = False

    ####       Write your set of step commands below       ###

    


    ###############################

    time.sleep(1)

    tello.send_rc_control(0,0,0,0)
    
    time.sleep(3)
    
    tello.land()

    Registering = False

##      Open Loop - End             ## 
######################################



############################
##          Plot          ##
############################

fichier = open("result.txt","r")

Time = []
ToF = []
cmd = []
consign = []
speed = []
for ligne in fichier:
    valeurs = ligne.split()
    Time.append(float(valeurs[0]))
    ToF.append(float(valeurs[1]))
    cmd.append(float(valeurs[2]))
    consign.append(float(valeurs[3]))
    speed.append(float(valeurs[4]))

fichier.close()

plt.plot(Time, ToF)
plt.plot(Time, cmd)
plt.plot(Time, consign)
plt.plot(Time, speed)

plt.xlabel('Temps')
plt.legend(["altitude","commande","consigne","vitesse sur l'axe Z"])
plt.show()

    